﻿#!/usr/bin/python
# -*- coding: utf-8 -*-
# game modul for minescape

"""game modul for minescape"""

# importing requiered moduls
import os,sys,pygame,time,random,pickle,eztext

# global vars
random.seed()
tile_group=None
player_group=None
bottom_group=None
exit_group=None
level_group=None
leveltext=None
player=None
exit=None
clues=None
clue_group=None
countdown=None
countdown_group=None
bgimg=None
gameoverscreen=None
gameover_sound=[]
gametools=None
gametools_group=None
score=0
scoretext=None
scoregroup=None

# loading global sounds and images
gameover_sound.append("placeholder")
gameover_sound.append(pygame.mixer.Sound(os.path.join("sounds","alarm.ogg")))
gameover_sound.append(pygame.mixer.Sound(os.path.join("sounds","boom.ogg")))
win_sound=pygame.mixer.Sound(os.path.join("sounds","win.ogg"))
win_sound.set_volume(0.7)
win_screen=pygame.image.load(os.path.join("graphics","win_screen.png")).convert_alpha()
win_screen_rect=win_screen.get_rect()
win_screen_rect.center=(640,384)

# creating sprite classes
class tile(pygame.sprite.Sprite):
    """Sprite class for the game board tile"""
    def __init__(self):
        "Init board tile sprite"
        self.tileimage=[]
        count=[0,1,2,3,4,5,6]
        for i in count:
            self.tileimage.append(pygame.image.load(os.path.join("graphics","tile"+str(i)+".png")).convert())
        pygame.sprite.Sprite.__init__(self)
        #state 0=default, 1=flag, 2=destroyed, 3=cleared, 4=obstacle, 5=bomb, 6=bomflag
        self.state=0
        #type 0=empty, 4=obstacle, 5=bomb
        self.type=0
        self.image=self.tileimage[0]
        self.rect=self.image.get_rect()
        self.dirty=0
    def update(self):
        """Update board tile sprite.
        If the sprite is "dirty" it refreshes its state"""
        if self.dirty:
            self.image=self.tileimage[self.state]
            self.dirty=0
    def clear(self):
        """Set the tile as cleared"""
        self.state=3
        self.dirty=1

class Player(pygame.sprite.Sprite):
    """Sprite class for the player"""
    def __init__(self,level):
        "Init player sprite, set some properties depending on the passed gamelevel"
        pygame.sprite.Sprite.__init__(self)
        self.image=pygame.image.load(os.path.join("graphics","player.png")).convert_alpha()
        self.bombed_image=pygame.image.load(os.path.join("graphics","player_boom.png")).convert_alpha()
        self.rect=self.image.get_rect()
        self.rect.topleft=(321,660)
        self.step=40
        self.x=1
        self.y=1
        self.dirty=0
        self.flag=9+level
        if level>2:
            self.dropweight=int(level/3)
            if self.dropweight>4:
                self.dropweight=4
        else:
            self.dropweight=0
        if level>4:
            self.digbot=int(level/5)
            if self.digbot>3:
                self.digbot=3
        else:
            self.digbot=0
        self.step_sound=pygame.mixer.Sound(os.path.join("sounds","step.ogg"))
        self.step_sound.set_volume(0.6)

    def move_up(self,tiles):
        "Move the player up with one field if possible"
        if self.y<15:
            if tiles[(self.x-1)*14+(15-(self.y+1))].type<>4:
                self.y+=1
                self.dirty=1
                self.step_sound.play()
    def move_down(self,tiles):
        "Move the player down with one field if possible"
        if self.y>1:
                if self.y>2 and tiles[(self.x-1)*14+(15-(self.y-1))].type==4:
                    return
                self.y-=1
                self.dirty=1
                self.step_sound.play()
    def move_left(self,tiles):
        "Move the player left with one field if possible"
        if self.x>1:
            if self.y>1 and tiles[((self.x-1)-1)*14+(15-self.y)].type==4:
                return
            self.x-=1
            self.dirty=1
            self.step_sound.play()
    def move_right(self,tiles):
        "Move the player right with one field if possible"
        if self.x<16:
            if self.y>1:
                if self.x<15 and tiles[((self.x+1)-1)*14+(15-self.y)].type==4:
                    return
            self.x+=1
            self.dirty=1
            self.step_sound.play()
    def update(self):
        "Update the players position if moved (self.dirty is True)"
        if self.dirty:
            self.rect.topleft=(281+self.x*self.step,700-self.y*self.step)
            self.dirty=0
    def isbombed(self,tiles):
        "Check if the player is standing on a bomb"
        if self.y==1:
            return 0
        i=(self.x-1)*14+(15-self.y)
        if tiles[i].type==5:
            self.image=self.bombed_image
            return 1
        else:
            return 0

class Exit(pygame.sprite.Sprite):
    """Sprite class for the exit sign"""
    def __init__(self):
        "Init the exit sprite"
        pygame.sprite.Sprite.__init__(self)
        self.x=1
        self.image=pygame.image.load(os.path.join("graphics","exit.png")).convert_alpha()
        self.rect=self.image.get_rect()
        self.rect.topleft=(321,60)
    def update(self):
        "Updates the exit sprite's position"
        self.rect.topleft=(281+40*self.x,60)

class Countdown(pygame.sprite.Sprite):
    """Sprite class for the countdown counter"""
    def _convtime(self,timeval):
        "Converting countdown time value to string"
        returnstring=""
        minutes=timeval/60
        strminutes="00"+str(minutes)
        strminutes=strminutes[-2:]
        strseconds="00"+str(timeval-60*minutes)
        strseconds=strseconds[-2:]
        returnstring=strminutes+":"+strseconds
        return returnstring
    def _render(self):
        "Rendering the countdown string"
        self.font=pygame.font.Font(os.path.join('graphics','DejaVuSans-Bold.ttf'),30)
        self.image=self.font.render(self.string,1,self.textcolor)
        self.rect=self.image.get_rect()
        self.rect.center=(128,73)
    def __init__(self,starttime):
        "Init countdown sprite. Starttime passed as a time value"
        pygame.sprite.Sprite.__init__(self)
        if starttime>5999: starttime=5999
        self.start=starttime
        self.acttime=starttime
        self.startT=int(round(time.time()))
        self.string=self._convtime(starttime)
        self.textcolor=[0,0,0]
        self._render()
        self.gameover=False
        self.clock_sound=pygame.mixer.Sound(os.path.join("sounds","clock.ogg"))
        self.clock2_sound=pygame.mixer.Sound(os.path.join("sounds","clock2.ogg"))

    def update(self):
        """Updating the countdown actual time and string.
        If it is 10 seconds or below, the color changes to red"""
        self.acttime=self.start-(int(round(time.time()))-self.startT)
        self.string=self._convtime(self.acttime)
        if self.acttime==10:
            self.textcolor=[255,0,0]
        if self.acttime==0:
            self.gameover=True
        self._render()

class Level(pygame.sprite.Sprite):
    """Sprite class to display game level"""
    def __init__(self):
        "Init game level indicator sprite"
        pygame.sprite.Sprite.__init__(self)
        self.level=1
        self.font=pygame.font.Font(os.path.join('graphics','DejaVuSans-Bold.ttf'),30)
        self.color=[0,0,255]
        self._render()

    def _render(self):
        "Render game level indicator sprite"
        self.image=self.font.render("LEVEL "+ str(self.level),1,self.color)
        self.rect=self.image.get_rect()
        self.rect.center=(128,704)

    def update(self,level):
        "Updating game level indicator"
        self.level=level
        self._render()

class GameTools(pygame.sprite.Sprite):
    "Sprite class for the player tools"
    def __init__(self,flags,dropweights,digbots):
        """Init gametools sprite properties and the sprite itself
        Passed arguments are the number of flags, dropweights and digbots"""
        pygame.sprite.Sprite.__init__(self)
        self.flags=flags
        self.dropweights=dropweights
        self.digbots=digbots
        self.y1=54
        self.y2=162
        self.y3=270
        self.font=pygame.font.Font(os.path.join('graphics','DejaVuSans-Bold.ttf'),24)
        self.textcolor=(0,0,0)
        #gametools window position: 25,159 size: 206,324
        self.window=pygame.Surface((206,324),pygame.SRCALPHA)
        self.window.fill((0,0,0,0))
        self.rect=self.window.get_rect()
        self.rect.topleft=(25,159)
        self.flag=pygame.image.load(os.path.join("graphics","flag.png")).convert_alpha()
        self.flagrect=self.flag.get_rect()
        self.flagrect.center=(40,self.y1)
        self.dropweight=pygame.image.load(os.path.join("graphics","dropweight.png")).convert_alpha()
        self.dropweightrect=self.dropweight.get_rect()
        self.dropweightrect.center=(40,self.y2)
        self.digbot=pygame.image.load(os.path.join("graphics","digbot.png")).convert_alpha()
        self.digbotrect=self.digbot.get_rect()
        self.digbotrect.center=(40,self.y3)
        self.line=self.font.render("--",1,self.textcolor)
        self.linerect=self.line.get_rect()
    def render(self):
        "Rendering the gametool window"
        # blit tool icons onto tool window
        self.window.blit(self.flag,self.flagrect)
        self.window.blit(self.dropweight,self.dropweightrect)
        self.window.blit(self.digbot,self.digbotrect)
        #blit lines onto tool window
        line_x=103
        no_x=166
        self.linerect.center=(line_x,self.y1)
        self.window.blit(self.line,self.linerect)
        self.linerect.center=(line_x,self.y2)
        self.window.blit(self.line,self.linerect)
        self.linerect.center=(line_x,self.y3)
        self.window.blit(self.line,self.linerect)
        # blit tool numbers onto tool window
        flag_no=self.font.render(str(self.flags),1,self.textcolor)
        flag_no_rect=flag_no.get_rect()
        flag_no_rect.center=(no_x,self.y1)
        dropweight_no=self.font.render(str(self.dropweights),1,self.textcolor)
        dropweight_no_rect=dropweight_no.get_rect()
        dropweight_no_rect.center=(no_x,self.y2)
        self.window.blit(dropweight_no,dropweight_no_rect)
        digbot_no=self.font.render(str(self.digbots),1,self.textcolor)
        digbot_no_rect=digbot_no.get_rect()
        digbot_no_rect.center=(no_x,self.y3)
        self.window.blit(flag_no,flag_no_rect)
        self.window.blit(dropweight_no,dropweight_no_rect)
        self.window.blit(digbot_no,digbot_no_rect)
        self.image=self.window
        # blit the tool window onto screen
        #screen.blit(self.window,self.rect)
    def update(self,flags,dropweights,digbots):
        "Updating the gametool properties"
        self.flags=flags
        self.dropweights=dropweights
        self.digbots=digbots
        self.window.fill((0,0,0,0))
        self.render()

class Clue(pygame.sprite.Sprite):
    """Sprite class to display the mine clues.
    These numbers indicates how many mine could be found beside a given tile"""
    def __init__(self,x,y):
        "Init clue sprite. x,y the coords of the tile the clue related to the player"
        pygame.sprite.Sprite.__init__(self)
        self.font=pygame.font.Font(os.path.join('graphics','DejaVuSans-Bold.ttf'),18)
        self.color=[[0,0,0],[0,0,255],[0,255,0],[255,0,0],[150,20,20],[238,0,255],[255,170,0],[70,0,160]]
        self.x=x
        self.y=y
        self.step=40

    def setclue(self,n,coords):
        """Set the clue. n is the clue number (see get_clue function) coords is 
        a list with the coordinates of the player sprite rect"""
        self.n=n
        self.image=self.font.render(str(n),1,self.color[n])
        self.rect=self.image.get_rect()
        self.rect.center=(coords[0]+self.x*self.step,coords[1]-self.y*self.step)

class gameover_screen():
    def __init__(self):
        self.gameover_screen=[]
        self.gameover_screen.append("placeholder")
        self.gameover_screen.append(pygame.image.load(os.path.join("graphics","game_over1.png")).convert_alpha())
        self.gameover_screen.append(pygame.image.load(os.path.join("graphics","game_over2.png")).convert_alpha())
        self.rect=self.gameover_screen[1].get_rect()
        self.upcoords=(640,284)
        self.lowcoords=(640,484)
    def display(self,gameover_type,y,screen):
        if y>7:
            coords=self.lowcoords
        else:
            coords=self.upcoords
        self.rect.center=coords
        screen.blit(self.gameover_screen[gameover_type],self.rect)

class BottomMessage():
    def __init__(self,message,color,bgcolor):
        self.font=pygame.font.Font(os.path.join('graphics','DejaVuSans-Bold.ttf'),16)
        self.message=" "+message+" "
        self.m_update(message,color,bgcolor)
    def m_update(self,message,color,bgcolor):
        self.image=self.font.render(self.message,1,color)
        self.bg=pygame.Surface(self.image.get_rect().inflate(20,10).size)
        self.bg.fill(bgcolor)
        self.image_rect=self.image.get_rect()
        self.bg_rect=self.bg.get_rect()
        self.image_rect.center=self.bg_rect.center
        self.bg.blit(self.image,self.image_rect)
        self.bg_rect.center=(640,680)
    def display(self,screen):
        screen.blit(self.bg,self.bg_rect)

class PutFlag(BottomMessage):
    def __init__(self,screen):
        BottomMessage.__init__(self,"PUT FLAG: Press an arrow to set the flag position",(250,250,85),(70,50,20))
        self.tick=0
        self.display(screen)
        self.buzzer=pygame.mixer.Sound(os.path.join("sounds","buzzer.ogg"))
        self.putsound=pygame.mixer.Sound(os.path.join("sounds","putflag.ogg"))
        self.flag_down=False
        self.score=0
    def update(self,screen,player,tiles,put_flag):
        self.tick+=1
        if self.tick>200:
            return False
        else:
            events=pygame.event.get()
            for event in events:
                if event.type == pygame.QUIT:
                    sys.exit()
                elif (event.type == pygame.KEYDOWN and event.key in [pygame.K_ESCAPE, pygame.K_q]):
                    return False
                elif (event.type==pygame.KEYDOWN and (event.key in [pygame.K_UP,pygame.K_DOWN,pygame.K_LEFT,pygame.K_RIGHT])):
                    if self.checkflag(player,tiles,event.key):
                        self.putflag(player,tiles,event.key)
                        return False
                    else:
                        self.invalidflag()
            return True
    def checkflag(self,player,tiles,eventkey):
        directiondict={pygame.K_UP:[0,1],pygame.K_DOWN:[0,-1],pygame.K_LEFT:[-1,0],pygame.K_RIGHT:[1,0]}
        direction=directiondict[eventkey]
        x=player.x+direction[0]
        y=player.y+direction[1]
        if x>0 and x<17 and y>1 and y<16:
            type=tiles[(x-1)*14+(15-y)].state
            if type<>1 and type<>4:
                return True
            else:
                return False
        else:
            return False
    def putflag(self,player,tiles,eventkey):
        directiondict={pygame.K_UP:[0,1],pygame.K_DOWN:[0,-1],pygame.K_LEFT:[-1,0],pygame.K_RIGHT:[1,0]}
        direction=directiondict[eventkey]
        x=player.x+direction[0]
        y=player.y+direction[1]
        n=(x-1)*14+(15-y)
        tiles[n].state=1
        tiles[n].dirty=1
        if tiles[n].type==5:
            self.score=5
        self.putsound.play()
        self.flag_down=True
        player.flag-=1
    def invalidflag(self):
        self.buzzer.play()

class DropWeight(BottomMessage):
    def __init__(self,screen):
        BottomMessage.__init__(self,"DROP WEIGHT: Press an arrow to set drop direction",(250,250,85),(70,50,20))
        self.tick=0
        self.display(screen)
        self.buzzer=pygame.mixer.Sound(os.path.join("sounds","buzzer.ogg"))
        self.dropsound=pygame.mixer.Sound(os.path.join("sounds","drop.ogg"))
        self.dropped=False
        self.weightimage=pygame.image.load(os.path.join("graphics","dropweight.png")).convert_alpha()
        self.weightrect=self.image.get_rect()
        self.score=0
    def update(self,screen,player,tiles,drop_weight):
        self.tick+=1
        if self.tick>200:
            return False
        else:
            events=pygame.event.get()
            for event in events:
                if event.type == pygame.QUIT:
                    sys.exit()
                elif (event.type == pygame.KEYDOWN and event.key in [pygame.K_ESCAPE, pygame.K_q]):
                    return False
                elif (event.type==pygame.KEYDOWN and (event.key in [pygame.K_UP,pygame.K_DOWN,pygame.K_LEFT,pygame.K_RIGHT])):
                    if self.checkpos(player,tiles,event.key):
                        self.drop(player,tiles,event.key,screen)
                        return False
                    else:
                        self.invalidpos()
            return True
    def checkpos(self,player,tiles,eventkey):
        directiondict={pygame.K_UP:[0,2],pygame.K_DOWN:[0,-2],pygame.K_LEFT:[-2,0],pygame.K_RIGHT:[2,0]}
        direction=directiondict[eventkey]
        x=player.x+direction[0]
        y=player.y+direction[1]
        if x>0 and x<17 and y>1 and y<16:
            type=tiles[(x-1)*14+(15-y)].type
            if type<>4:
                return True
            else:
                return False
        else:
            return False
    def drop(self,player,tiles,eventkey,screen):
        directiondict={pygame.K_UP:[0,2],pygame.K_DOWN:[0,-2],pygame.K_LEFT:[-2,0],pygame.K_RIGHT:[2,0]}
        direction=directiondict[eventkey]
        x=player.x+direction[0]
        y=player.y+direction[1]
        for xi in [-1,0,1]:
            for yi in [-1,0,1]:
                xm=x+xi
                ym=y+yi
                if xm>0 and xm<17 and ym>1 and ym<16:
                    n=(xm-1)*14+(15-ym)
                    if tiles[n].type<>4:
                        if tiles[n].type==5:
                            self.score+=5
                        tiles[n].type=0
                        tiles[n].state=2
                        tiles[n].dirty=1
        self.dropped=True
        player.dropweight-=1
        self.weightrect=tiles[(x-1)*14+(15-y)].rect
        self.dropsound.play()
        screen.blit(self.weightimage,self.weightrect)
        pygame.display.flip()
        pygame.time.wait(200)
    def invalidpos(self):
        self.buzzer.play()

class DigBot(BottomMessage):
    def __init__(self,screen):
        BottomMessage.__init__(self,"DIGBOT: Digbot released.",(250,250,85),(70,50,20))
        self.tick=0
        self.buzzer=pygame.mixer.Sound(os.path.join("sounds","buzzer.ogg"))
        self.digsound=pygame.mixer.Sound(os.path.join("sounds","digbot.ogg"))
        self.boomsound=pygame.mixer.Sound(os.path.join("sounds","boom.ogg"))
        self.released=False
        self.bombed=False
        self.image=pygame.image.load(os.path.join("graphics","digbot.png")).convert_alpha()
        self.image_bombed=pygame.image.load(os.path.join("graphics","digbot_bombed.png")).convert_alpha()
        self.rect=self.image.get_rect()
        self.x=0
        self.y=0
        self.score=0
    def update(self,screen,player,tiles,dig_bot):
        self.tick+=1
        if self.released==False:
            if not self.checkpos(player,tiles):
                self.invalidpos()
                return False
            else:
                self.released=True
                player.digbot-=1
                self.set_pos(player)
                self.digsound.play(-1)
                return self.movebot(tiles)
        else:
            if self.tick%40==0:
                if not self.bombed:
                    return self.movebot(tiles)
                else:
                    return False
            else:
                return True
    def checkpos(self,player,tiles):
        if tiles[(player.x-1)*14+(15-player.y-1)].type==4:
            return False
        else:
            return True
    def set_pos(self,player):
        self.x=player.x
        self.y=1
    def invalidpos(self):
        self.buzzer.play()
    def movebot(self,tiles):
        if self.y>14:
            self.digsound.stop()
            return False
        else:
            self.y+=1
            n=(self.x-1)*14+(15-self.y)
            self.rect.topleft=tiles[n].rect.topleft
            if tiles[n].type==5:
                self.score=5
                self.image=self.image_bombed
                self.digsound.stop()
                self.bombed=True
                self.boomsound.play()
                tiles[n].type=0
                tiles[n].state=3
                tiles[n].dirty=1
                return True
            else:
                tiles[n].state=3
                tiles[n].dirty=1
                return True
    def render(self,screen):
        self.display(screen)
        screen.blit(self.image,self.rect)

class Score(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        self.font=pygame.font.Font(os.path.join('graphics','DejaVuSans-Bold.ttf'),30)
        self.color=(255,55,20)
    def update(self,score):
        self.image=self.font.render("SCORE: "+str(score),1,self.color)
        self.rect=self.image.get_rect()
        self.rect.center=(128,580)
class HighScores():
    def __init__(self):
        self.highscores=[]
        if not os.path.isfile("highscores.bin"):
            self.inithighscores()
        file=open("highscores.bin","rb")
        self.highscores=pickle.load(file)
        file.close()
        self.bigfont=pygame.font.Font(os.path.join('graphics','DejaVuSans-Bold.ttf'),24)
        self.smallfont=pygame.font.Font(os.path.join('graphics','DejaVuSans-Bold.ttf'),16)
    def inithighscores(self):
        self.highscores.append(["Barney",2000])
        self.highscores.append(["Giselle",1800])
        self.highscores.append(["Digbot",1400])
        self.highscores.append(["Arnold",1200])
        self.highscores.append(["Hacker",1000])
        self.highscores.append(["Sergei",800])
        self.highscores.append(["Osama",600])
        self.highscores.append(["Superman",500])
        self.highscores.append(["Warrior",400])
        self.highscores.append(["Python",250])
        file=open("highscores.bin","w")
        pickle.dump(self.highscores,file)
        file.close()
    def checkscore(self,score):
        item=9
        if self.highscores[9][1]>score:
            return False
        while item>=0:
            print item, self.highscores[item][0], self.highscores[item][1],score
            if self.highscores[item][1]>score:
                return item+1
            item-=1
        return False
    def insertscore(self,name,score,item):
        if item==9:
            self.highscores[9][0]=name
            self.highscores[9][1]=score
        else:
            i=9
            while i>item:
                self.highscores[i]=self.highscores[i-1]
                i-=1
            self.highscores[item]=[name,score]
        file=open("highscores.bin","w")
        pickle.dump(self.highscores,file)
        file.close()
    def getname(self,screen):
        outerbox=pygame.surface.Surface((360,150))
        outerrect=outerbox.get_rect()
        outerbox.fill((200,100,10))
        outerrect.center=screen.get_rect().center
        title=self.bigfont.render("You have a high score!",1,(255,255,255))
        titlerect=title.get_rect()
        titlerect.center=(180,25)
        outerbox.blit(title,titlerect)
        innerbox=pygame.surface.Surface((300,50))
        innerrect=innerbox.get_rect()
        innerrect.center=(180,90)
        innerbox.fill((255,245,105))
        outerbox.blit(innerbox,innerrect)
        inputbox=eztext.Input(x=15,y=12,maxlength=8, color=(255,0,0), prompt='Type your name: ')
        innerclock=pygame.time.Clock()
        exit=False
        while not exit:
            innerclock.tick(30)
            events = pygame.event.get()
            for event in events:
                if event==pygame.QUIT:
                    sys.exit()
                if event.type==pygame.KEYDOWN and event.key==pygame.K_RETURN:
                    exit=True
            inputbox.update(events)
            innerbox.fill((255,245,105))
            inputbox.draw(innerbox)
            outerbox.blit(innerbox,innerrect)
            screen.blit(outerbox,outerrect)
            pygame.display.flip()
        return inputbox.value
def get_clue(player,clue,tiles):
    x=player.x+clue.x
    y=player.y+clue.y
    clues=0
    if x>0 and x<17 and y>1 and y<16:
        for xi in [-1,0,1]:
            for yi in [-1,0,1]:
                xm=x+xi
                ym=y+yi
                if  not(xi==0 and yi==0):
                    if xm>0 and xm<17 and ym>1 and ym<16:
                        if tiles[(xm-1)*14+(15-ym)].type==5:
                            clues+=1
    return clues

def generate_clues():
    global clues, player, tiles, clue_group
    clue_group.empty()
    for item in clues:
        item.setclue(get_clue(player,item,tiles),player.rect.center)
        if item.n<>0:
            clue_group.add(item)

def init_gamedata(level):
    global tiles, tile_group,player_group,bottom_row,bottom_group,player,exit,exit_group, \
    countdown,countdown_group, leveltext,level_group, clues, clue_group,gametools,gametools_group, score, scoretext, score_group
    tiles=[]
    bottom_row=[]
    for x in range (16):
        r=tile()
        r.state=3
        r.dirty=1
        r.update()
        r.rect.topleft=(321+x*40,660)
        bottom_row.append(r)
        for y in range(14):
            t=tile()
            t.rect.topleft=(321+x*40,100+y*40)
            tiles.append(t)
    tile_group=pygame.sprite.RenderPlain(*tiles)
    bottom_group=pygame.sprite.RenderPlain(*bottom_row)
    player=Player(level)
    player_group=pygame.sprite.RenderPlain(player)
    exit=Exit()
    exit.x=random.randint(1,16)
    exit.update()
    exit_group=pygame.sprite.RenderPlain(exit)
    countdown=Countdown(45+level*15)
    countdown_group=pygame.sprite.RenderPlain(countdown)
    leveltext=Level()
    leveltext.update(level)
    level_group=pygame.sprite.RenderPlain(leveltext)
    scoretext=Score()
    scoretext.update(score)
    score_group=pygame.sprite.RenderPlain(scoretext)
    clues=[Clue(-1,-1),Clue(-1,0),Clue(-1,1),Clue(0,1),Clue(1,1),Clue(1,0),Clue(1,-1),Clue(0,-1)]
    clue_group=pygame.sprite.RenderPlain()
    generate_clues()
    gametools=GameTools(player.flag,player.dropweight,player.digbot)
    gametools.render()
    gametools_group=pygame.sprite.RenderPlain(gametools)
    #set level dependent parameters
    bombs=12+level*2
    if level<3:
        obstacles=0
    else:
        obstacles=int(level/3)
    #set tiles
    steplist=[]
    prevstep=[0,0]
    tilelist=[]
    for x in range (16):
        for y in range(14):
            tilelist.append([x,y])

    step=[exit.x-1,0]
    tilelist.remove(step)

    while step[1]<13:
        if (step[0]>1 and prevstep<>(1,0)): steplist.append([-1,0])
        if (step[0]>15 and prevstep<>(-1,0)): steplist.append([1,0])
        if step[1]<14: steplist.append([0,1])
        prevstep=steplist[random.randint(0,len(steplist)-1)]
        step=[step[0]+prevstep[0],step[1]+prevstep[1]]
        tilelist.remove(step)
        steplist=[]
    if len(tilelist)<(bombs+obstacles):
        bombs=len(tilelist)-obstacles
    while bombs>0:
        random_element=random.randint(0,len(tilelist)-1)
        actual_tile=tilelist[random_element]
        tiles[actual_tile[0]*14+actual_tile[1]].type=5
        tilelist.remove(actual_tile)
        bombs=bombs-1

    while obstacles>0:
        random_element=random.randint(0,len(tilelist)-1)
        actual_tile=tilelist[random_element]
        index=actual_tile[0]*14+actual_tile[1]
        tiles[index].type=4
        tiles[index].state=4
        tiles[index].dirty=1
        tilelist.remove(actual_tile)
        obstacles=obstacles-1

def background(screen):
    global bgimg
    bgfile=os.path.join('graphics',"board.png")
    bgimg=pygame.image.load(bgfile).convert()
    bgrect=bgimg.get_rect()
    screen.blit(bgimg,bgrect)
    pygame.display.flip()
    #game area 668x668 topleft 306,50


def init_gamescreen(screen):
    #show_level1()
    background(screen)
    tile_group.draw(screen)
    bottom_group.draw(screen)
    player_group.draw(screen)
    exit_group.draw(screen)
    countdown_group.draw(screen)
    level_group.draw(screen)
    clue_group.draw(screen)
    gametools_group.draw(screen)
    score_group.draw(screen)
    pygame.display.flip()

def game_render(screen):
    tile_group.draw(screen)
    bottom_group.draw(screen)
    player_group.draw(screen)
    clue_group.draw(screen)
    pygame.display.flip()

def game_render2(screen):
    tile_group.draw(screen)
    bottom_group.draw(screen)
    player_group.draw(screen)
    clue_group.draw(screen)

def game_countdown(screen):
    countdown_group.clear(screen,bgimg)
    countdown_group.update()
    countdown_group.draw(screen)
    pygame.display.flip()

def gameover_action(n,y,screen,gameoverscreen):
    gameover_sound[n].play()
    gameoverscreen.display(n,y,screen)
    pygame.display.flip()
    pressanykey()

def show_level(screen):
    for item in tiles:
        if item.type<>0:
            if item.type==5 and item.state==1:
                item.state=6
            else:
                item.state=item.type
            item.dirty=1

    tile_group.update()
    game_render(screen)
    pygame.display.flip()

def show_level1():
    for item in tiles:
        if item.type<>0:
            item.state=item.type
            item.dirty=1
    tile_group.update()

def win_level(screen):
    show_level(screen)
    screen.blit(win_screen,win_screen_rect)
    pygame.display.flip()
    win_sound.play()
    pressanykey()

def pressanykey():
    while 1:
        pygame.event.clear()
        event=pygame.event.wait()
        if event.type == pygame.KEYDOWN:
            break

def start(screen):
    global score
    level=1
    clock=pygame.time.Clock()
    gameoverscreen=gameover_screen()
    score=0
    Highscores=HighScores()
    while 1:
        init_gamedata(level)
        level_win=0
        game_over=0
        boom=0
        clockwarning=0
        tile_group.update()
        countdown.clock_sound.play(-1)
        init_gamescreen(screen)
        put_flag=False
        drop_weight=False
        run_digbot=False
        putflag_loop=None
        dropweight_loop=None
        digbot_loop=None
        while level_win==0:
            if not put_flag and not drop_weight and not run_digbot:
                events=pygame.event.get()
                for event in events:
                    if event.type == pygame.QUIT:
                        sys.exit()
                    elif (event.type == pygame.KEYDOWN and event.key in [pygame.K_ESCAPE, pygame.K_q]):
                        action="EXIT"
                        countdown.clock_sound.stop()
                        countdown.clock2_sound.stop()
                        return action
                    elif (event.type==pygame.KEYDOWN and event.key==pygame.K_UP):
                        if (player.y==15 and exit.x==player.x):
                            level_win=1
                            countdown.clock_sound.stop()
                            countdown.clock2_sound.stop()
                        player.move_up(tiles)
                        boom=player.isbombed(tiles)
                        if not boom and player.y>1:
                            tiles[(player.x-1)*14+(15-player.y)].clear()
                        player.update()
                        generate_clues()
                        game_render(screen)
                    elif (event.type==pygame.KEYDOWN and event.key==pygame.K_DOWN):
                        player.move_down(tiles)
                        boom=player.isbombed(tiles)
                        if not boom and player.y>1:
                            tiles[(player.x-1)*14+(15-player.y)].clear()
                        player.update()
                        generate_clues()
                        game_render(screen)
                    elif (event.type==pygame.KEYDOWN and event.key==pygame.K_LEFT):
                        player.move_left(tiles)
                        boom=player.isbombed(tiles)
                        if not boom and player.y>1:
                            tiles[(player.x-1)*14+(15-player.y)].clear()
                        player.update()
                        generate_clues()
                        game_render(screen)
                    elif (event.type==pygame.KEYDOWN and event.key==pygame.K_RIGHT):
                        player.move_right(tiles)
                        boom=player.isbombed(tiles)
                        if not boom and player.y>1:
                            tiles[(player.x-1)*14+(15-player.y)].clear()
                        player.update()
                        generate_clues()
                        game_render(screen)
                    elif (event.type==pygame.KEYDOWN and event.key==pygame.K_f and player.flag>0):
                        put_flag=True
                        putflag_loop=PutFlag(screen)
                    elif (event.type==pygame.KEYDOWN and event.key==pygame.K_d and player.dropweight>0):
                        drop_weight=True
                        dropweight_loop=DropWeight(screen)
                    elif (event.type==pygame.KEYDOWN and event.key==pygame.K_b and player.digbot>0 and player.y==1):
                        run_digbot=True
                        digbot_loop=DigBot(screen)
                if boom==1:
                    game_over=2
                    break
            else:
                if putflag_loop<>None:
                    put_flag=putflag_loop.update(screen,player,tiles,put_flag)
                    if put_flag==False:
                        tile_group.update()
                        game_render(screen)
                        if putflag_loop.flag_down==True:
                            score+=putflag_loop.score
                            scoretext.update(score)
                            score_group.clear(screen,bgimg)
                            score_group.draw(screen)
                            gametools.update(player.flag,player.dropweight,player.digbot)
                            gametools_group.clear(screen,bgimg)
                            gametools_group.draw(screen)
                            putflag_loop=None
                if dropweight_loop<>None:
                    drop_weight=dropweight_loop.update(screen,player,tiles,drop_weight)
                    if drop_weight==False:
                        tile_group.update()
                        generate_clues()
                        game_render(screen)
                        if dropweight_loop.dropped==True:
                            score+=dropweight_loop.score
                            scoretext.update(score)
                            score_group.clear(screen,bgimg)
                            score_group.draw(screen)
                            gametools.update(player.flag,player.dropweight,player.digbot)
                            gametools_group.clear(screen,bgimg)
                            gametools_group.draw(screen)
                            dropweight_loop=None
                if digbot_loop<>None:
                    run_digbot=digbot_loop.update(screen,player,tiles,run_digbot)
                    if run_digbot==False:
                        tile_group.update()
                        game_render(screen)
                        gametools.update(player.flag,player.dropweight,player.digbot)
                        gametools_group.clear(screen,bgimg)
                        gametools_group.draw(screen)
                        score+=digbot_loop.score
                        scoretext.update(score)
                        score_group.clear(screen,bgimg)
                        score_group.draw(screen)
                        digbot_loop=None
                    else:
                        tile_group.update()
                        game_render2(screen)
                        digbot_loop.render(screen)
                        pygame.display.flip()

            game_countdown(screen)
            if countdown.gameover:
                game_over=1
                break
            if countdown.acttime==10 and clockwarning==0:
                countdown.clock_sound.stop()
                countdown.clock2_sound.play(-1)
                clockwarning=1
            clock.tick(40)
            tile_group.update()
        countdown.clock_sound.stop()
        countdown.clock2_sound.stop()
        if game_over<>0:
            show_level(screen)
            game_render(screen)
            scoretext.update(score)
            score_group.clear(screen,bgimg)
            score_group.draw(screen)
            gameover_action(game_over,player.y,screen,gameoverscreen)
            n=Highscores.checkscore(score)
            print "highscorecheck: ",n
            if n<>False:
                name=Highscores.getname(screen)
                Highscores.insertscore(name,score,n)
            return
        if level_win:
            win_level(screen)
            score+=100
            scoretext.update(score)
            score_group.clear(screen,bgimg)
            score_group.draw(screen)
        level=level+1